var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import React from "react";

const IS_SCROLLING_DEBOUNCE_INTERVAL = 150;

export function createListComponent({
  getCellSize,
  getEstimatedTotalSize,
  getOffsetForIndex,
  getStartIndexForOffset,
  getStopIndexForStartIndex,
  validateProps
}) {
  var _class, _temp2;

  return _temp2 = _class = class List extends React.Component {
    constructor(...args) {
      var _temp;

      return _temp = super(...args), this._cellStyleCache = {}, this._resetIsScrollingTimeoutId = null, this.state = {
        isScrolling: false,
        scrollDirection: "forward",
        scrollOffset: 0
      }, this.onScrollHorizontal = event => {
        const { scrollLeft } = event.currentTarget;
        this.setState(prevState => ({
          isScrolling: true,
          scrollDirection: prevState.scrollOffset < scrollLeft ? "forward" : "backward",
          scrollOffset: scrollLeft
        }), this.resetIsScrollingDebounced);
      }, this.onScrollVertical = event => {
        const { scrollTop } = event.currentTarget;
        this.setState(prevState => ({
          isScrolling: true,
          scrollDirection: prevState.scrollOffset < scrollTop ? "forward" : "backward",
          scrollOffset: scrollTop
        }), this.resetIsScrollingDebounced);
      }, this.scrollingContainerRef = ref => {
        this._scrollingContainer = ref;
      }, this.resetIsScrollingDebounced = () => {
        if (this._resetIsScrollingTimeoutId !== null) {
          clearTimeout(this._resetIsScrollingTimeoutId);
        }

        this._resetIsScrollingTimeoutId = setTimeout(this.resetIsScrolling, IS_SCROLLING_DEBOUNCE_INTERVAL);
      }, this.resetIsScrolling = () => {
        this._resetIsScrollingTimeoutId = null;

        this.setState({ isScrolling: false }, () => {
          // Clear style cache after state update has been committed.
          // This way we don't break pure sCU for cells that don't use isScrolling param.
          this._cellStyleCache = {};
        });
      }, _temp;
    }

    static getDerivedStateFromProps(nextProps, prevState) {
      validateSharedProps(nextProps);
      validateProps(nextProps);
      return null;
    }

    scrollTo(scrollOffset) {
      if (this._scrollingContainer != null) {
        const { direction } = this.props;

        if (direction === "horizontal") {
          this._scrollingContainer.scrollLeft = scrollOffset;
        } else {
          this._scrollingContainer.scrollTop = scrollOffset;
        }
      }
    }

    scrollToRow(index, align = "auto") {
      if (this._scrollingContainer != null) {
        const { scrollOffset } = this.state;
        this.scrollTo(getOffsetForIndex(this.props, index, align, scrollOffset));
      }
    }

    componnetWillUnmount() {
      if (this._resetIsScrollingTimeoutId !== null) {
        clearTimeout(this._resetIsScrollingTimeoutId);
      }
    }

    render() {
      const { className, direction, height, style, width } = this.props;
      const { isScrolling } = this.state;

      const onScroll = direction === "vertical" ? this.onScrollVertical : this.onScrollHorizontal;

      const estimatedTotalSize = getEstimatedTotalSize(this.props);

      return React.createElement(
        "div",
        {
          className: className,
          ref: this.scrollingContainerRef,
          style: _extends({
            position: "relative",
            height,
            width,
            overflow: "auto"
          }, style),
          onScroll: onScroll
        },
        React.createElement(
          "div",
          {
            style: {
              height: direction === "horizontal" ? height : estimatedTotalSize,
              overflow: "hidden",
              pointerEvents: isScrolling ? "none" : "",
              width: direction === "horizontal" ? estimatedTotalSize : width
            }
          },
          this.renderCells()
        )
      );
    }

    renderCells() {
      const { children, direction, useIsScrolling } = this.props;
      const { isScrolling } = this.state;

      const [startIndex, stopIndex] = this.getRangeToRender();

      const cells = [];

      for (let index = startIndex; index <= stopIndex; index++) {
        const key = "" + index;

        // Cache cell styles while scrolling,
        // So that pure component sCU will prevent re-renders.
        let style;
        if (this._cellStyleCache.hasOwnProperty(index)) {
          style = this._cellStyleCache[index];
        } else {
          // TODO Get position of cell using helper, not hard-coded number type
          this._cellStyleCache[index] = style = {
            position: "absolute",
            left: direction === "horizontal" ? index * getCellSize(this.props, index) : 0,
            top: direction === "vertical" ? index * getCellSize(this.props, index) : 0,
            height: direction === "vertical" ? getCellSize(this.props, index) : "100%",
            width: direction === "horizontal" ? getCellSize(this.props, index) : "100%"
          };
        }

        cells.push(children({
          key,
          index,
          isScrolling: useIsScrolling ? isScrolling : undefined,
          style
        }));
      }

      return cells;
    }

    getRangeToRender() {
      const { count, overscanCount } = this.props;
      const { scrollDirection, scrollOffset } = this.state;

      const startIndex = getStartIndexForOffset(this.props, scrollOffset);
      const stopIndex = getStopIndexForStartIndex(this.props, startIndex);

      // Overscan by one cell in each direction so that tab/focus works.
      // If there isn't at least one extra cell, tab loops back around.
      const overscanBackward = scrollDirection === "backward" ? Math.max(1, overscanCount) : 1;
      const overscanForward = scrollDirection === "forward" ? Math.max(1, overscanCount) : 1;

      return [Math.max(0, startIndex - overscanBackward), Math.min(count - 1, stopIndex + overscanForward)];
    }

  }, _class.defaultProps = {
    direction: "vertical",
    overscanCount: 2,
    useIsScrolling: false
  }, _temp2;
}

const validateSharedProps = ({
  children,
  direction,
  height,
  width
}) => {
  if (direction !== "horizontal" && direction !== "vertical") {
    throw Error('An invalid "direction" prop has been specified. ' + 'Value should be either "horizontal" or "vertical". ' + `"${direction}" was specified.`);
  }

  if (typeof children !== "function") {
    throw Error('An invalid "children" prop has been specified. ' + "Value should be a function that creates a React element. " + `"${children === null ? "null" : typeof children}" was specified.`);
  }

  if (direction === "horizontal" && typeof width !== "number") {
    throw Error('An invalid "width" prop has been specified. ' + "Horizontal lists must specify a number for width. " + `"${width === null ? "null" : typeof width}" was specified.`);
  } else if (direction === "vertical" && typeof height !== "number") {
    throw Error('An invalid "height" prop has been specified. ' + "Vertical lists must specify a number for height. " + `"${height === null ? "null" : typeof height}" was specified.`);
  }
};